#
# $Id: PloneContactFolder.py 22138 2005-07-20 10:46:41Z kilobug $
#

"""PloneContactFolder class"""

# Standard Python imports
import string, os, re, time

# Zope core imports
from AccessControl import ClassSecurityInfo, Unauthorized
from Products.Archetypes.BaseFolder import BaseFolder
from AccessControl.SecurityManagement import newSecurityManager, getSecurityManager, setSecurityManager

from Products.Archetypes.ArchetypeTool import registerType
from Products.Archetypes.Schema import getSchemata
from Products.Archetypes.Renderer import renderer

from Products.PloneContactFolder.PloneContactItem import PloneContactItem

from Products.CMFCore.CMFCorePermissions import setDefaultRoles

# New specific permissions
ViewContacts = 'View Contacts'

# Default roles for those permissions
setDefaultRoles(ViewContacts, ('Manager',))

# CMF/Plone imports
from Products.CMFCore.CMFCorePermissions import View, ListFolderContents, ModifyPortalContent

DEFAULT_TEMPLATE = """Subject: %(title)s\r
\r
This is a message sent from the contact form of the site %(portal_name)s.\r
\r
Title: %(person_title)s\r
Name: %(name)s\r
From: %(from)s\r
Company: %(company)s\r
Category: %(category)s\r
Message:\r
%(message)s
"""

def _cmpfunc(item1, item2):
    if (item2.created() > item1.created()):
        return 1
    elif (item2.created() < item1.created()):
        return -1
    else:
        return 0

def _reversedcmp(item1, item2):
    if item1 > item2:
        return -1
    elif item2 > item1:
        return 1
    else:
        return 0

#########################################
## The PloneContactFolder content type ##
##########################################

class PloneContactFolder(BaseFolder):
    """PloneContactFolder class
    """

    ########################################
    ## Registration info for portal_types ##
    ########################################

    schema=BaseFolder.schema

    meta_type = portal_type = archetype_name = 'Plone Contact Folder'

    content_icon = 'PloneContactFolder.png'

    filter_content_types = 1
    
    allowed_content_types = ('Plone Contact Item',)

    actions = (
            {'id': 'view',
             'name': 'View',
             'action': 'string:${object_url}/contactfolder_view',
             'permissions': (ViewContacts,)},

            {'id': 'folderlisting',
             'name': 'Folder Listing',
             'action': 'string:${object_url}/folder_contents',
             'permissions': (ListFolderContents,)},
            
            {'id': 'edit',
             'name': 'Edit',
             'action': 'string:${object_url}/contactfolder_edit_form',
             'permissions': (ModifyPortalContent,)},

            {'id': 'metadata',
             'name': 'Properties',
             'action': 'string:${object_url}/metadata_edit_form',
             'permissions': (ModifyPortalContent,)},
        )

    _mailAddrRx = re.compile(r"^([0-9a-zA-Z_&.+-]+!)*[0-9a-zA-Z_&.+-]"
                             r"+@(([0-9a-z]([0-9a-z-]*[0-9a-z])?\.)"
                             r"+[a-z]{2,4}|([0-9]{1,3}\.){3}[0-9]{1,3})$")

    ###########################
    ## Basic class behaviour ##
    ###########################


    # Standard security settings
    security = ClassSecurityInfo()

    # Init method
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        BaseFolder.__init__(self, id, title = title)
        self.categories = {}
        self.template = DEFAULT_TEMPLATE

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, title='', categories = {}, template = ''):
        """Changes values"""
        user = self.REQUEST.AUTHENTICATED_USER
        if not user.has_permission(ModifyPortalContent, self):
            raise Unauthorized        
        self.title = title
        self.categories = categories
        if template:
            self.template = template
        self.reindexObject()

    security.declareProtected(ViewContacts, "getContents")
    def getContents(self, categories = None, month = None):
        """Get all objects of given categories and given months"""
        user = self.REQUEST.AUTHENTICATED_USER
        if not user.has_permission(ViewContacts, self):
            raise Unauthorized        
        objects = self.objectValues()
        if not categories is None:
            objects = [ obj for obj in objects if obj.category in categories ]
        if not month is None:
            objects = [ obj for obj in objects if obj.getMonth() == month ]
        objects.sort(_cmpfunc)
        return objects

    security.declareProtected(ViewContacts, "getMonths")
    def getMonths(self):
        """Get all available months"""
        user = self.REQUEST.AUTHENTICATED_USER
        if not user.has_permission(ViewContacts, self):
            raise Unauthorized        
        objects = self.objectValues()
        months = [ obj.getMonth() for obj in objects ]
        nmonths = []
        for month in months:
            if not month in nmonths:
                nmonths.append(month)
        months = nmonths
        # months = list(set(months)) # Python 2.4 only :/
        months.sort(_reversedcmp)
        return months

    security.declareProtected(ViewContacts, "getCategories")
    def getCategories(self):
        """Get all categories

        Categories are a dictionnary with the id as key, and a values
        dictionnary as value. The keys of the values dictionnaries are
        'name' for the name, and 'mails' for the email addresses.

        Example:
        {
          'cat1': { 'name': 'General',
                    'mails': [ 'contact@foo.org', 'info@foo.org' ] },
          'cat2': { 'name': 'Technical Support',
                    'mails': [ 'contact@foo.org', 'support@foo.org' ] }
        }                   
        """
        user = self.REQUEST.AUTHENTICATED_USER
        if not user.has_permission(ViewContacts, self):
            raise Unauthorized        
        return self.categories
    
    security.declareProtected(ViewContacts, "getTemplate")
    def getTemplate(self):
        """Get the template"""        
        user = self.REQUEST.AUTHENTICATED_USER
        if not user.has_permission(ViewContacts, self):
            raise Unauthorized        
        return getattr(self, "template", DEFAULT_TEMPLATE)
    
    security.declareProtected(ModifyPortalContent, "setCategories")
    def setCategories(self, categories):
        """Set all categories"""
        user = self.REQUEST.AUTHENTICATED_USER
        if not user.has_permission(ModifyPortalContent, self):
            raise Unauthorized        
        self.categories = categories
    
    security.declarePublic('checkMailAddress')
    def checkMailAddress(self, someAddr):
        """Checks the validity of a mail address"""
        return self._mailAddrRx.match(someAddr.strip()) != None

    security.declarePublic('newContactItem')
    def newContactItem(self, title='', name='', mailfrom='',
                       company='', category='', message=''):
        """Create a new object and send it"""
        new_id = "contact_%f" % time.time()
        oldsm = getSecurityManager()
        newSecurityManager(self.REQUEST, self.ownerOfObject())
        self.invokeFactory(type_name='Plone Contact Item', id=new_id)
        obj = getattr(self, new_id)
        obj.edit(title=title, name=name, mailfrom=mailfrom,
                 company=company, category=category, message=message)
        obj.send()
        setSecurityManager(oldsm)

    security.declarePrivate('ownerOfObject')
    def ownerOfObject(self):
        """Provides acl_user acquisition wrapped owner of object"""
        udb, uid = self.getOwner(1)
        root = self.getPhysicalRoot()
        udb = root.unrestrictedTraverse(udb, None)
        if udb is None:
            user = SpecialUsers.nobody
        else:
            user = udb.getUserById(uid, None)
            if user is None:
                user = SpecialUsers.nobody
            else:
                user = user.__of__(udb)
        return user

    security.declarePublic('getCategoryList')
    def getCategoryList(self):
        """Get the list of category"""
        categories = self.categories
        lst = [ (id,values["name"]) for id,values in categories.items() ]
        return lst

    security.declarePublic('getPersonTitles')
    def getPersonTitles(self):
        """Get the list of titles"""
        lst = [ ('Ms', 'Ms'), ('Mr', 'Mr'), ('Dr', 'Dr'), ('Pr', 'Pr') ]
        return lst

# Class instanciation
registerType(PloneContactFolder)

